'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _virtex = require('virtex');

var _evStore = require('ev-store');

var _evStore2 = _interopRequireDefault(_evStore);

var _events = require('./events');

var _events2 = _interopRequireDefault(_events);

var _toStyle = require('to-style');

var _foreach = require('@micro-js/foreach');

var _foreach2 = _interopRequireDefault(_foreach);

var _isObject = require('@micro-js/is-object');

var _isObject2 = _interopRequireDefault(_isObject);

var _keychord = require('@micro-js/keychord');

var _keychord2 = _interopRequireDefault(_keychord);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; } /**
                                                                                                                              * Imports
                                                                                                                              */

/**
 * virtex-element
 */

function element(tag, attrs) {
  // Only apply sugar to native elements
  if (typeof tag === 'string' && attrs) {
    (0, _foreach2.default)(function (val, key) {
      return attrs[key] = sugar(val, key);
    }, attrs);
  }

  return _virtex.element.apply(null, arguments);
}

function sugar(value, name) {
  return styleSugar(eventSugar(focusSugar(classSugar(value, name), name), name), name);
}

function styleSugar(value, name) {
  if (name === 'style' && (0, _isObject2.default)(value)) {
    return (0, _toStyle.string)(value);
  }

  return value;
}

function eventSugar(value, name) {
  return _events2.default.hasOwnProperty(name) ? bindEvent(_events2.default[name], value) : value;
}

function focusSugar(value, name) {
  return name === 'focused' ? focusElement : value;
}

function classSugar(value, name) {
  if (name === 'class') {
    if (Array.isArray(value)) {
      return value.join(' ');
    } else if ((0, _isObject2.default)(value)) {
      var _ret = (function () {
        var str = '';

        (0, _foreach2.default)(function (val, key) {
          if (val) {
            if (str) str += ' ';
            str += key;
          }
        }, value);

        return {
          v: str
        };
      })();

      if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
    }
  }

  return value;
}

function bindEvent(eventName, fn) {
  return function (node) {
    return (0, _evStore2.default)(node)[eventName] = createHandler(eventName, fn);
  };
}

function createHandler(eventName, fn) {
  if ((0, _isObject2.default)(fn) && isKeyboardEvent(eventName)) {
    return function (e) {
      var chord = (0, _keychord2.default)(e);
      if (fn[chord]) {
        var f = fn[chord];
        return Array.isArray(f) ? f.map(function (f) {
          return f(e);
        }) : f(e);
      }
    };
  } else if (Array.isArray(fn)) {
    return function (e) {
      return fn.map(function (f) {
        return f(e);
      });
    };
  } else {
    return fn;
  }
}

function isKeyboardEvent(eventName) {
  return ['keydown', 'keypress', 'keyup'].indexOf(eventName) !== -1;
}

function focusElement(node) {
  setTimeout(function () {
    if (node.ownerDocument.activeElement !== node) {
      node.focus();
    }
  });
}

/**
 * Exports
 */

exports.default = element;